#include <linux/ioport.h>
#include <asm/io.h>
#include <string.h>
#include "version.h"

#define LCD_DATA 0x378
#define LCD_CMD  LCD_DATA+2
#define LCD_LEFT   0
#define LCD_CENTER 1
#define LCD_RIGHT  2

/***********************
 ***********************
 * LCD commands        *
 ***********************
 ***********************/

/******************************
 * Execute a LCD command byte *
 ******************************/
void LCD_WriteCTRL(char command)
{
  outb(3,LCD_CMD);        /* RS=0, RW=0, E=0  (bin 11) */
  outb(command,LCD_DATA); /* Send command    */
  outb(2,LCD_CMD);        /* RS=0, RW=0, E=1  (bin 10) */
  outb(3,LCD_CMD);        /* RS=0, RW=0, E=0  (bin 11) */
  outb(1,LCD_CMD);        /* RS=0, RW=1, E=0  (bin 01) */
  usleep(40);
}

/*************************
 * Write a LCD character *
 *************************/
void LCD_WriteData(char data)
{
  outb(7,LCD_CMD);
  outb(data,LCD_DATA);
  outb(6,LCD_CMD);
  outb(7,LCD_CMD);
  outb(5,LCD_CMD);
  usleep(40);
}


/********************************
 * Reset LCD panel and set mode *
 ********************************/
void init_lcd(void)
{
  if (ioperm(LCD_DATA,3,1)!=0) /* Open hardware... do you have permission ? */
    {
      printf("\n(C) 1999 Hans Luyten, LCD demo v0.9\n");
      printf("\nERROR : Missing privileges, unable to open hardware port.\n");
      exit(1);
    }

  usleep(15000);  /* After Power On wait 15 ms */
  LCD_WriteCTRL(0x38); /* 0011xxxx for 8-bit mode */ 
  usleep(4100);   /* wait 4.1ms */
  LCD_WriteCTRL(0x38); 
  usleep(100);    /* wait 100us */
  LCD_WriteCTRL(0x38);
  usleep(4100);   /* wait 4.1ms */

 /* These next lines should be used, however it works without aswell */
 /* LCD_WriteCTRL(0x08); /* Display & Cursor & Blink all OFF */
 /* usleep(40);     /* wait 40us */
 /* LCD_WriteCTRL(0x0c); /* Display ON, Cursor & Blink OFF */
 /* usleep(1640);   /* wait 1.64ms */

  LCD_WriteCTRL(0x06); /* Entry mode: to right, no display shift */
  usleep(40);     /* wait 40us */ 
  LCD_WriteCTRL(0x0c);
  usleep(40);
  LCD_WriteCTRL(0x01); /* Clear display */
  usleep(1640);   /* wait 1.64ms */
  LCD_WriteCTRL(0x02); /* Cursor home */
  usleep(1640);   /* Wait 1.64ms */
}

void LCD_Clear()
{
  LCD_WriteCTRL(0x01);
  usleep(1640);
}

void LCD_Home()
{
  LCD_WriteCTRL(2);
}

void LCD_Position(char row, char column)
{
  switch(row) {
    case 1: LCD_WriteCTRL(0x80+column-1); break;
    case 2: LCD_WriteCTRL(0xc0+column-1); break;
    case 3: LCD_WriteCTRL(0x94+column-1); break;
    case 4: LCD_WriteCTRL(0xd4+column-1); break;
    default: break;
    }
}

int LCD_WriteRow(char string[], int length, int row, int align)
{
  int i;
  unsigned char j;

  if (length==0) length=strlen(string);
  
  if (align==LCD_LEFT) LCD_Position(row,1);
  if (align==LCD_RIGHT) LCD_Position(row,21-length);
  if (align==LCD_CENTER) LCD_Position(row,(21-length)/2);
 
  for (i=0;i<length;i++)
  {
     j=*(string+i);
     if (j=='\0') return(0);
     LCD_WriteData(j);
  }
 
  return(0); 
}

void main()
{
  init_lcd();     
  LCD_WriteRow("Hallo1234",0,3,LCD_CENTER);
}
